@@ -1,23 +1,15 @@
 #include <Rcpp.h>
 using namespace Rcpp;
 
-// This is a simple example of exporting a C++ function to R. You can
-// source this function into an R session using the Rcpp::sourceCpp 
-// function (or via the Source button on the editor toolbar). Learn
-// more about Rcpp at:
-//
-//   http://www.rcpp.org/
-//   http://adv-r.had.co.nz/Rcpp.html
-//   http://gallery.rcpp.org/
-//
+
 
 NumericMatrix scoregaussian(NumericVector beta, NumericVector Y, NumericMatrix DataM, NumericVector weight, NumericVector offset) {
   int i,j;
   int nrow=DataM.nrow();
   int ncol=DataM.ncol();
-  NumericMatrix out(nrow,ncol);
+  NumericMatrix out(nrow,ncol+1);
   double eta;
-
+  
   for(i = 0; i < nrow; ++i)
   { eta=0;
     for (j = 0; j < ncol; ++j){
@@ -25,8 +17,9 @@ NumericMatrix scoregaussian(NumericVector beta, NumericVector Y, NumericMatrix D
     }
     eta+=offset(i);
     for (j = 0; j < ncol; ++j){
-      out(i,j) = weight(i)*(Y(i)-eta)*DataM(i,j);
+      out(i,j) = weight(i)*(Y(i)-eta)*DataM(i,j)/pow(beta(ncol),2);
     }
+    out(i,ncol) = weight(i) * (- 1/beta(ncol) + pow(Y(i)-eta,2)/pow(beta(ncol),3));
   }
   
   return out;
@@ -37,13 +30,13 @@ NumericVector scoregaussiancpp(NumericVector beta, NumericVector Y, NumericMatri
   int i,j;
   int nrow=DataM.nrow();
   int ncol=DataM.ncol();
-  NumericMatrix scorez0(nrow,ncol),scorez1(nrow,ncol);
-  NumericVector value(ncol);
+  NumericMatrix scorez0(nrow,ncol+1),scorez1(nrow,ncol+1);
+  NumericVector value(ncol+1);
   
   scorez0=scoregaussian(beta,Y,DataM0,weight,offset);
   scorez1=scoregaussian(beta,Y,DataM1,weight,offset);
   
-  for (j = 0; j < ncol; ++j){
+  for (j = 0; j < ncol+1; ++j){
     value(j)=0;
     for(i = 0; i < nrow; ++i){
       value(j)+=((1-DataM(i,ncol-1))*(scorez0(i,j)*(1-phat(i))-scorez1(i,j)*qhat(i))-DataM(i,ncol-1)*(scorez0(i,j)*phat(i)-scorez1(i,j)*(1-qhat(i))))/(1-phat(i)-qhat(i));
@@ -53,10 +46,6 @@ NumericVector scoregaussiancpp(NumericVector beta, NumericVector Y, NumericMatri
   return value;
 }
 
-// You can include R code blocks in C++ files processed with sourceCpp
-// (useful for testing and development). The R code will be automatically 
-// run after the compilation.
-//
 
 /*** R
 
